﻿using iTextSharp.text;
using iTextSharp.text.pdf;
using System.Collections;
using System.IO;
using System;
using System.Collections.Generic;

public class MergeEx
{
    #region Fields
    private string sourcefolder;
    private string destinationfile;
    private IList fileList = new ArrayList();
    #endregion

    #region Public Methods
    ///
    /// Add a new file, together with a given docname to the fileList and namelist collection
    ///
    public void AddFile(string pathnname)
    {
        fileList.Add(pathnname);
    }

    ///
    /// Generate the merged PDF
    ///
    public void Execute()
    {
        MergeDocs();
    }
    #endregion

    #region Private Methods
    ///
    /// Merges the Docs and renders the destinationFile
    ///
    private void MergeDocs()
    {
        

        //Step 1: Create a Docuement-Object
        Document document = new Document();
        try
        {
            //Step 2: we create a writer that listens to the document
            PdfWriter writer = PdfWriter.GetInstance(document, new FileStream(destinationfile, FileMode.Create));

            //Step 3: Open the document
            document.Open();

            PdfContentByte cb = writer.DirectContent;
            PdfImportedPage page;

            int n = 0;
            int rotation = 0;

            //Loops for each file that has been listed
            foreach (string filename in fileList)
            {
                //The current file path
                string filePath = sourcefolder + filename;

                // we create a reader for the document
                PdfReader reader = new PdfReader(filePath);

                //Gets the number of pages to process
                n = reader.NumberOfPages;
                
                int i = 0;
                while (i < n)
                {
                    i++;
                    document.SetPageSize(reader.GetPageSizeWithRotation(1));
                    document.NewPage();

                    //Insert to Destination on the first page
                    if (i == 1)
                    {
                        Chunk fileRef = new Chunk(" ");
                        fileRef.SetLocalDestination(filename);
                        document.Add(fileRef);
                    }

                    page = writer.GetImportedPage(reader, i);
                    rotation = reader.GetPageRotation(i);
                    if (rotation == 90 || rotation == 270)
                    {
                        cb.AddTemplate(page, 0, -1f, 1f, 0, 0, reader.GetPageSizeWithRotation(i).Height);
                    }
                    else
                    {
                        cb.AddTemplate(page, 1f, 0, 0, 1f, 0, 0);
                    }
                }
            }
        }
        catch (Exception e) { throw e; }
        finally { document.Close(); }
    }


    //public static void AppendToDocument(string sourcePdfPath1, string sourcePdfPath2, string outputPdfPath)
    //{
    //    using (var sourceDocumentStream1 = new FileStream(sourcePdfPath1, FileMode.Open))
    //    {
    //        using (var sourceDocumentStream2 = new FileStream(sourcePdfPath2, FileMode.Open))
    //        {
    //            using (var destinationDocumentStream = new FileStream(outputPdfPath, FileMode.Create))
    //            {
    //                var pdfConcat = new PdfConcatenate(destinationDocumentStream);
    //                var pdfReader = new PdfReader(sourceDocumentStream1);

    //                var pages = new List<int>();
    //                for (int i = 0; i < pdfReader.NumberOfPages; i++)
    //                {
    //                    pages.Add(i);
    //                }

    //                pdfReader.SelectPages(pages);
    //                pdfConcat.AddPages(pdfReader);

    //                pdfReader = new PdfReader(sourceDocumentStream2);

    //                pages = new List<int>();
    //                for (int i = 0; i < pdfReader.NumberOfPages; i++)
    //                {
    //                    pages.Add(i);
    //                }

    //                pdfReader.SelectPages(pages);
    //                pdfConcat.AddPages(pdfReader);

    //                pdfReader.Close();
    //                pdfConcat.Close();
    //            }
    //        }
    //    }
    //}

    //public void addnumeropagina {
    
    //        PdfImportedPage page;
    //        PdfContentByte contentByte = writer.DirectContent;                

    //        int rotation;
    //        while (sourceIndex < sourceFileList.Count)
    //        {
    //                        int pageIndex = 0;
    //                        while (pageIndex < sourceFilePageCount)
    //                        {
    //                            pageIndex++;

    //                            doc.SetPageSize(reader.GetPageSizeWithRotation(pageIndex));
    //                            doc.NewPage();

    //                            page = writer.GetImportedPage(reader, pageIndex);
    //                            rotation = reader.GetPageRotation(pageIndex);

    //                            if (rotation.Equals(90 | 270))
    //                                contentByte.AddTemplate(page, 0, -1f, 1f, 0, 0, reader.GetPageSizeWithRotation(pageIndex).Height);
    //                            else
    //                                contentByte.AddTemplate(page, 1f, 0, 0, 1f, 0, 0);
    //                        }

    //                        sourceIndex++;
    //                        if (sourceIndex < sourceFileList.Count)
    //                        {
    //                            reader = new PdfReader(sourceFileList[sourceIndex]);
    //                            sourceFilePageCount = reader.NumberOfPages;
    //                        }
    //        }

    //        doc.Close();
    
    //}

    //public static byte[] AddPageNumbers(byte[] pdf)
    //{
    //    MemoryStream ms = new MemoryStream();
    //    ms.Write(pdf, 0, pdf.Length);
    //    // we create a reader for a certain document
    //    PdfReader reader = new PdfReader(pdf);
    //    // we retrieve the total number of pages
    //    int n = reader.NumberOfPages;
    //    // we retrieve the size of the first page
    //    Rectangle psize = reader.GetPageSize(1);

    //    // step 1: creation of a document-object
    //    Document document = new Document(psize, 50, 50, 50, 50);
    //    // step 2: we create a writer that listens to the document
    //    PdfWriter writer = PdfWriter.GetInstance(document, ms);
    //    // step 3: we open the document

    //    document.Open();
    //    // step 4: we add content
    //    PdfContentByte cb = writer.DirectContent;

    //    int p = 0;
    //    Console.WriteLine("There are " + n + " pages in the document.");
    //    for (int page = 1; page <= reader.NumberOfPages; page++)
    //    {
    //        document.NewPage();
    //        p++;

    //        PdfImportedPage importedPage = writer.GetImportedPage(reader, page);
    //        cb.AddTemplate(importedPage, 0, 0);

    //        BaseFont bf = BaseFont.CreateFont(BaseFont.HELVETICA, BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
    //        cb.BeginText();
    //        cb.SetFontAndSize(bf, 10);
    //        cb.ShowTextAligned(PdfContentByte.ALIGN_LEFT, +p + "/" + n, 7, 44, 0);
    //        cb.EndText();
    //    }
    //    // step 5: we close the document
    //    document.Close();
    //    return ms.ToArray();
    //}

    #endregion

    #region Properties
    ///
    /// Gets or Sets the SourceFolder
    ///
    public string SourceFolder
    {
        get { return sourcefolder; }
        set { sourcefolder = value; }
    }

    ///
    /// Gets or Sets the DestinationFile
    ///
    public string DestinationFile
    {
        get { return destinationfile; }
        set { destinationfile = value; }
    }
    #endregion
}